#include "stdafx.h"
#include "HDUSTest.h"
#include "StatusView.h"


#define STATUS_WINDOW_CLASS	TEXT("HDUSTest Status")

#define STATUS_MARGIN	4




void CStatusItem::SetStatusView(CStatusView *pStatus)
{
	m_pStatus=pStatus;
}


int CStatusItem::GetIndex() const
{
	int i;

	for (i=0;i<m_pStatus->NumItems();i++) {
		if (m_pStatus->GetItem(i)==this)
			return i;
	}
	return -1;
}


bool CStatusItem::GetRect(RECT *pRect) const
{
	int Index=GetIndex();

	if (Index<0)
		return false;
	m_pStatus->GetItemRect(Index,pRect);
	return true;
}




HINSTANCE CStatusView::m_hinst=NULL;


bool CStatusView::Initialize(HINSTANCE hinst)
{
	WNDCLASS wc;

	m_hinst=hinst;
	wc.style=0;
	wc.lpfnWndProc=WndProc;
	wc.cbClsExtra=0;
	wc.cbWndExtra=0;
	wc.hInstance=hinst;
	wc.hIcon=NULL;
	wc.hCursor=LoadCursor(NULL,IDC_ARROW);
	wc.hbrBackground=NULL;
	wc.lpszMenuName=NULL;
	wc.lpszClassName=STATUS_WINDOW_CLASS;
	return RegisterClass(&wc)!=0;
}


CStatusView::CStatusView()
{
	m_hwnd=NULL;
	m_hfontStatus=NULL;
	m_crBackColor=RGB(128,192,160);
	m_crTextColor=RGB(64,96,80);
	m_crHighlightBackColor=RGB(64,96,80);
	m_crHighlightTextColor=RGB(128,192,160);
	m_NumItems=0;
	m_fSingleMode=false;
	m_pszSingleText=NULL;
	m_HotItem=-1;
	m_fTrackMouseEvent=false;
}


CStatusView::~CStatusView()
{
	if (m_hwnd!=NULL)
		DestroyWindow(m_hwnd);
	for (int i=0;i<m_NumItems;i++)
		delete m_pItemList[i];
	delete [] m_pszSingleText;
}


bool CStatusView::Create(HWND hwndParent,int ID,DWORD Style,DWORD ExStyle)
{
	m_hwnd=CreateWindowEx(ExStyle,STATUS_WINDOW_CLASS,NULL,Style,
									0,0,0,0,hwndParent,(HMENU)ID,m_hinst,this);
	return m_hwnd!=NULL;
}


const CStatusItem *CStatusView::GetItem(int Index) const
{
	if (Index<0 || Index>=m_NumItems)
		return NULL;
	return m_pItemList[Index];
}


bool CStatusView::AddItem(CStatusItem *pItem)
{
	if (m_NumItems>=MAX_STATUS_ITEMS)
		return false;
	m_pItemList[m_NumItems++]=pItem;
	pItem->SetStatusView(this);
	return true;
}


CStatusView *CStatusView::GetStatusView(HWND hwnd)
{
	return (CStatusView*)GetWindowLongPtr(hwnd,GWLP_USERDATA);
}


LRESULT CALLBACK CStatusView::WndProc(HWND hwnd,UINT uMsg,WPARAM wParam,
																LPARAM lParam)
{
	switch (uMsg) {
	case WM_CREATE:
		{
			LPCREATESTRUCT pcs=(LPCREATESTRUCT)lParam;
			CStatusView *pStatus;
			LOGFONT lf;
			HDC hdc;
			HFONT hfontOld;
			TEXTMETRIC tm;
			RECT rc;

			pStatus=(CStatusView*)pcs->lpCreateParams;
			SetWindowLongPtr(hwnd,GWLP_USERDATA,(LONG_PTR)pStatus);
			GetObject(GetStockObject(DEFAULT_GUI_FONT),sizeof(LOGFONT),&lf);
			pStatus->m_hfontStatus=CreateFontIndirect(&lf);
			hdc=GetDC(hwnd);
			hfontOld=(HFONT)SelectObject(hdc,pStatus->m_hfontStatus);
			GetTextMetrics(hdc,&tm);
			SelectObject(hdc,hfontOld);
			ReleaseDC(hwnd,hdc);
			rc.left=0;
			rc.top=0;
			rc.right=0;
			rc.bottom=tm.tmHeight+STATUS_MARGIN*2;
			AdjustWindowRectEx(&rc,pcs->style,FALSE,pcs->dwExStyle);
			MoveWindow(hwnd,0,0,0,rc.bottom-rc.top,FALSE);
		}
		return 0;

	case WM_PAINT:
		{
			CStatusView *pStatus=GetStatusView(hwnd);
			PAINTSTRUCT ps;
			HBRUSH hbrBk,hbrHighlight;
			HFONT hfontOld;
			COLORREF crOldTextColor,crOldBkColor;
			int OldBkMode;
			RECT rc;
			TCHAR szText[64];
			int i;
			int x;

			BeginPaint(hwnd,&ps);
			hfontOld=(HFONT)SelectObject(ps.hdc,pStatus->m_hfontStatus);
			OldBkMode=SetBkMode(ps.hdc,TRANSPARENT);
			hbrBk=CreateSolidBrush(pStatus->m_crBackColor);
			if (pStatus->m_HotItem>=0)
				hbrHighlight=CreateSolidBrush(pStatus->m_crHighlightBackColor);
			else
				hbrHighlight=NULL;
			crOldTextColor=GetTextColor(ps.hdc);
			crOldBkColor=GetBkColor(ps.hdc);
			GetClientRect(hwnd,&rc);
			if (pStatus->m_fSingleMode) {
				FillRect(ps.hdc,&ps.rcPaint,hbrBk);
				SetTextColor(ps.hdc,pStatus->m_crTextColor);
				rc.left+=STATUS_MARGIN;
				DrawText(ps.hdc,pStatus->m_pszSingleText,-1,&rc,
						DT_LEFT | DT_SINGLELINE | DT_VCENTER | DT_NOPREFIX);
			} else {
				for (i=0;i<pStatus->m_NumItems;i++) {
					SetTextColor(ps.hdc,i==pStatus->m_HotItem?
						pStatus->m_crHighlightTextColor:pStatus->m_crTextColor);
					SetBkColor(ps.hdc,i==pStatus->m_HotItem?
						pStatus->m_crHighlightBackColor:pStatus->m_crBackColor);
					rc.right=rc.left+pStatus->m_pItemList[i]->Width()+
															STATUS_MARGIN*2;
					FillRect(ps.hdc,&rc,i==pStatus->m_HotItem?hbrHighlight:hbrBk);
					rc.left+=STATUS_MARGIN;
					rc.right-=STATUS_MARGIN;
					pStatus->m_pItemList[i]->Draw(ps.hdc,&rc);
					rc.left=rc.right+STATUS_MARGIN;
				}
				rc.right=ps.rcPaint.right;
				if (rc.right>rc.left)
					FillRect(ps.hdc,&rc,hbrBk);
			}
			DeleteObject(hbrBk);
			if (hbrHighlight!=NULL)
				DeleteObject(hbrHighlight);
			SetBkColor(ps.hdc,crOldBkColor);
			SetTextColor(ps.hdc,crOldTextColor);
			SetBkMode(ps.hdc,OldBkMode);
			SelectObject(ps.hdc,hfontOld);
			EndPaint(hwnd,&ps);
		}
		return 0;

	case WM_MOUSEMOVE:
		if (GetCapture()==hwnd) {
			CStatusView *pStatus=GetStatusView(hwnd);
			int x=GET_X_LPARAM(lParam),y=GET_Y_LPARAM(lParam);
			RECT rc;

			pStatus->GetItemRect(pStatus->m_HotItem,&rc);
			x-=rc.left;
			pStatus->m_pItemList[pStatus->m_HotItem]->OnMouseMove(x,y);
		} else {
			CStatusView *pStatus=GetStatusView(hwnd);
			int x=GET_X_LPARAM(lParam);
			int Left,Right;
			int i;

			Left=0;
			for (i=0;i<pStatus->m_NumItems;i++) {
				Right=Left+STATUS_MARGIN*2+pStatus->m_pItemList[i]->Width();
				if (x>=Left && x<Right)
					break;
				Left=Right;
			}
			if (i==pStatus->m_NumItems)
				i=-1;
			if (i!=pStatus->m_HotItem) {
				int OldHotItem;

				OldHotItem=pStatus->m_HotItem;
				pStatus->m_HotItem=i;
				if (OldHotItem>=0)
					pStatus->UpdateItem(OldHotItem);
				if (pStatus->m_HotItem>=0)
					pStatus->UpdateItem(pStatus->m_HotItem);
			}
			if (!pStatus->m_fTrackMouseEvent) {
				TRACKMOUSEEVENT tme;

				tme.cbSize=sizeof(TRACKMOUSEEVENT);
				tme.dwFlags=TME_LEAVE;
				tme.hwndTrack=hwnd;
				if (TrackMouseEvent(&tme))
					pStatus->m_fTrackMouseEvent=true;
			}
		}
		return 0;

	case WM_MOUSELEAVE:
		{
			CStatusView *pStatus=GetStatusView(hwnd);

			if (pStatus->m_HotItem>=0) {
				int i=pStatus->m_HotItem;

				pStatus->m_HotItem=-1;
				pStatus->UpdateItem(i);
			}
			pStatus->m_fTrackMouseEvent=false;
		}
		return 0;

	case WM_LBUTTONDOWN:
	case WM_RBUTTONDOWN:
		{
			CStatusView *pStatus=GetStatusView(hwnd);

			if (pStatus->m_HotItem>=0) {
				int x=GET_X_LPARAM(lParam),y=GET_Y_LPARAM(lParam);
				RECT rc;

				pStatus->GetItemRect(pStatus->m_HotItem,&rc);
				x-=rc.left;
				if (uMsg==WM_LBUTTONDOWN)
					pStatus->m_pItemList[pStatus->m_HotItem]->OnLButtonDown(x,y);
				else
					pStatus->m_pItemList[pStatus->m_HotItem]->OnRButtonDown(x,y);
			}
		}
		return 0;

	case WM_LBUTTONUP:
		if (GetCapture()==hwnd) {
			ReleaseCapture();
		}
		return 0;

	case WM_SETCURSOR:
		if (LOWORD(lParam)==HTCLIENT) {
			CStatusView *pStatus=GetStatusView(hwnd);

			if (pStatus->m_HotItem>=0) {
				SetCursor(LoadCursor(NULL,IDC_HAND));
				return TRUE;
			}
		}
		break;

	case WM_DESTROY:
		{
			CStatusView *pStatus=GetStatusView(hwnd);

			DeleteObject(pStatus->m_hfontStatus);
			pStatus->m_hwnd=NULL;
		}
		return 0;
	}
	return DefWindowProc(hwnd,uMsg,wParam,lParam);
}


void CStatusView::UpdateItem(int Index)
{
	RECT rc;

	GetItemRect(Index,&rc);
	InvalidateRect(m_hwnd,&rc,TRUE);
}


bool CStatusView::GetItemRect(int Index,RECT *pRect) const
{
	int i;
	RECT rc;

	GetClientRect(m_hwnd,&rc);
	for (i=0;i<Index;i++)
		rc.left+=m_pItemList[i]->Width()+STATUS_MARGIN*2;
	rc.right=rc.left+m_pItemList[i]->Width()+STATUS_MARGIN*2;
	*pRect=rc;
	return true;
}


void CStatusView::GetRect(RECT *pRect) const
{
	GetWindowRect(m_hwnd,pRect);
}


int CStatusView::Height() const
{
	RECT rc;

	GetWindowRect(m_hwnd,&rc);
	return rc.bottom-rc.top;
}


void CStatusView::Move(int x,int y,int Width,int Height)
{
	MoveWindow(m_hwnd,x,y,Width,Height,TRUE);
}


void CStatusView::Show(bool fShow)
{
	ShowWindow(m_hwnd,fShow?SW_SHOW:SW_HIDE);
}


void CStatusView::SetSingleText(LPCTSTR pszText)
{
	delete [] m_pszSingleText;
	if (pszText!=NULL) {
		m_pszSingleText=new TCHAR[lstrlen(pszText)+1];
		lstrcpy(m_pszSingleText,pszText);
		m_fSingleMode=true;
	} else {
		m_pszSingleText=NULL;
		m_fSingleMode=false;
	}
	InvalidateRect(m_hwnd,NULL,TRUE);
}


void CStatusView::SetColor(COLORREF crBack,COLORREF crText,
							COLORREF crHighlightBack,COLORREF crHighlightText)
{
	m_crBackColor=crBack;
	m_crTextColor=crText;
	m_crHighlightBackColor=crHighlightBack;
	m_crHighlightTextColor=crHighlightText;
	InvalidateRect(m_hwnd,NULL,TRUE);
}
